function out=PreProcess_Genes(IndexLearn,DATA,LABELS,Apply,varargin)

%%  Last Modification : November, 15 November 2006.
%%  (a pre-processing box for standard Microarrays data set).
%%
%% INPUT variables
%%%%%%%%%%%%%%%%%%%
%%  IndexLearn ; matrix nLearn x 1
%%      indes of samples forming the learning set
%%  DATA : matrix n x pmax,  n >=nLearn
%%      contains the expression levels of all the genes in ALL the data set
%%  LABELS : matrix nLearn x 1
%%      LABELS of the samples in the LEARNING set : {0, ...,NbrClass-1}-valued
%%  
%%  Apply : matrix 1 x 5
%%      four binary variables to indicate what kind of pre-process has to
%%      be applied.
%%      Apply(1)=1 iff Thresholding has to be applied.
%%      Apply(2)=1 iff Filtering has to be applied.
%%      Apply(3)=1 iff Log-Transform has to be applied.
%%      Apply(4)=1 iff Standardization per row has to be applied.
%%      Apply(5)=1 iff Centring  per column has to be applied.
%%
%%  Optional input :
%%      DataSet : a character string
%%                 either 'Colon','Leukemia','Lymphoma','Prostate'.
%%                 defines some parameters for the Thresholding and
%%                 Filtering steps.
%%
%%
%%  OUTPUT variables
%%%%%%%%%%%%%%%%%%%%%
%%  A structure with fields :
%%      GENES : matrix n x pmax 
%%          
%% This m-file adjusts slightly an m-file provided by Dr Gersende Fort.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  STEP 1 : 
%%      TEST ON THE INPUT VARIABLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Number of Input variables
    if nargin<4,
        disp(sprintf('From PreProcess.m : \n \t Not enough input arguments'));
        out=[];
        return;
    end;
   
    if nargin==5;
        
        DataSet=varargin;        
    end
  
    
    if nargin==5 && ~strcmp(DataSet,'Colon') && ~strcmp(DataSet,'Lymphoma') && ~strcmp(DataSet,'Leukemia') && ~strcmp(DataSet,'Prostate'),
        disp(sprintf('From PreProcess.m : \n \t The DataSet is not known and the default parameters are used'));
        DataSet='Default';
    end;
    if (nargin==4 && sum(Apply(1:2))>0),
        disp(sprintf('From PreProcess.m : \n \t The default parameters are used'));
        DataSet='Default';
    end;
%   Size of DATA and LABELS
    if length(LABELS)~=size(DATA,1)
        disp(sprintf('From PreProcess : \n \t Error in the input variables, check the size of "%s" and "%s"',inputname(1),inputname(2)));
        out=[]; 
        return;
    end;
    
%%%%%%%%%%%%%%%%%%%%%%%%
%%  STEP 2 : 
%%       PRE-PROCESSING
%%%%%%%%%%%%%%%%%%%%%%%%
%% TRESHOLD
if Apply(1)==1,
    % Floor of  100 and Ceil at 16000
    if strcmp(DataSet,'Leukemia') || strcmp(DataSet,'Colon') || strcmp(DataSet,'Default')
        DATA=(DATA<100)*100+(DATA>=100).*DATA;
        DATA=(DATA>=16000)*16000+(DATA<16000).*DATA;  
    elseif strcmp(DataSet,'Lymphoma')
        DATA=(DATA<20)*20+(DATA>=20).*DATA; 
        DATA=(DATA>=16000)*16000+(DATA<16000).*DATA; 
    elseif strcmp(DataSet,'Prostate')
        DATA=(DATA<10)*10+(DATA>=10).*DATA; 
        DATA=(DATA>=16000)*16000+(DATA<16000).*DATA; 
    end;
end;

%%  FILTERING
DATAL=DATA(IndexLearn,:);
if Apply(2)==1,
    MaxCol=max(DATAL,[],1);
    MinCol=min(DATAL,[],1);
    if strcmp(DataSet,'Leukemia') || strcmp(DataSet,'Colon') || strcmp(DataSet,'Default')
        index=find(MaxCol./MinCol>5 & MaxCol-MinCol>500);
    elseif strcmp(DataSet,'Prostate')
        index=find(MaxCol./MinCol>5 & MaxCol-MinCol>50);
    end;    
    GENES=DATA(:,index);
else
    GENES=DATA;
end;

%%  LOG10-TRANSFORM
if Apply(3)==1,
    GENES=log10(GENES);
end;

%%  STANDARDIZATION     
if Apply(4)==1,
    pInit=size(GENES,2);
    MeanRow=mean(GENES,2);
    StdRow=std(GENES,1,2);
    GENES=(GENES-repmat(MeanRow,1,pInit))./repmat(StdRow,1,pInit);
        % now, the mean of each row is 0; and the sum of the square is
        % the length of each row;
end;

%% CENTRING
if Apply(5)==1;
    n=size(GENES,1);
    t=ones(n,1);
    GENES=(eye(n)-1/n*t*t')*GENES;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  STEP 3 :
%%      CONCLUSION
%%%%%%%%%%%%%%%%%%%%%%%%%%
out=GENES;